<?php

namespace Drupal\drd;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\drd\Entity\Core;
use Drupal\drd\Entity\Domain;
use Drupal\drd\Entity\Host;

/**
 * Query for DRD entities.
 */
class Entities implements EntitiesInterface {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * DRD entity type id.
   *
   * @var string
   */
  protected $type;

  /**
   * Properties for selection.
   *
   * @var array
   */
  protected $properties = [];

  /**
   * DRD host entity id.
   *
   * @var int
   */
  protected $hostId;

  /**
   * DRD core entity id.
   *
   * @var int
   */
  protected $coreId;

  /**
   * DRD domain entity id.
   *
   * @var int
   */
  protected $domainId;

  /**
   * Construct the Entity object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function setHostId($id) {
    $this->hostId = $id;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setCoreId($id) {
    $this->coreId = $id;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setDomainId($id) {
    $this->domainId = $id;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function hosts() {
    $this->type = 'host';
    $this->properties = [
      'status' => 1,
    ];
    if ($this->domainId) {
      /* @var \Drupal\drd\Entity\DomainInterface $domain */
      if ($domain = Domain::load($this->domainId)) {
        $this->coreId = $domain->getCore()->id();
      }
      else {
        return $this->none();
      }
    }
    if ($this->coreId) {
      /* @var \Drupal\drd\Entity\CoreInterface $core */
      if ($core = Core::load($this->coreId)) {
        $this->hostId = $core->getHost()->id();
      }
      else {
        return $this->none();
      }
    }
    if ($this->hostId) {
      return $this->search($this->hostId);
    }
    return $this->search();
  }

  /**
   * {@inheritdoc}
   */
  public function cores() {
    $this->type = 'core';
    $this->properties = [
      'status' => 1,
    ];
    if ($this->hostId) {
      $this->properties['host'] = $this->hostId;
    }
    else {
      if ($this->domainId) {
        /* @var \Drupal\drd\Entity\DomainInterface $domain */
        if ($domain = Domain::load($this->domainId)) {
          $this->coreId = $domain->getCore()->id();
        }
        else {
          return $this->none();
        }
      }
      if ($this->coreId) {
        return $this->search($this->coreId);
      }
    }
    return $this->search();
  }

  /**
   * {@inheritdoc}
   */
  public function domains() {
    $this->type = 'domain';
    $this->properties = [
      'status' => 1,
      'installed' => 1,
    ];
    if ($this->hostId) {
      if ($host = Host::load($this->hostId)) {
        array_walk($host->getCores(), function (Core $core) {
          $this->properties['core'][] = $core->id();
        });
      }
      else {
        return $this->none();
      }
    }
    elseif ($this->coreId) {
      $this->properties['core'] = $this->coreId;
    }
    elseif ($this->domainId) {
      return $this->search($this->domainId);
    }
    return $this->search();
  }

  /**
   * Nothing found, output a message and return FALSE.
   *
   * @return bool
   *   Retrun FALSE if no entity was found.
   */
  protected function none() {
    drupal_set_message('No ' . $this->type . ' found!', 'error');
    return FALSE;
  }

  /**
   * One entity found.
   *
   * @param int $id
   *   The id of the found entity.
   *
   * @return \Drupal\Core\Entity\EntityInterface[]
   *   Array containing one entity.
   */
  protected function oneEntity($id) {
    $entity = $this->entityTypeManager
      ->getStorage('drd_' . $this->type)
      ->load($id);
    return empty($entity) ? [] : [$entity];

  }

  /**
   * Multiple entities found.
   *
   * @param int $id
   *   ID of the entity to search or NULL to find all.
   *
   * @return bool|\Drupal\Core\Entity\EntityInterface[]
   *   List of all found entities.
   */
  protected function search($id = NULL) {
    $entities = isset($id) ? $this->oneEntity($id) :
      $this->entityTypeManager
        ->getStorage('drd_' . $this->type)
        ->loadByProperties($this->properties);

    if (empty($entities)) {
      return $this->none();
    }

    return $entities;
  }

}
